// Hlavickovy soubor obsahujici vstupne vystupni objekty jako napr cout
#include <iostream>

// Hlavickovy soubor obsahujici funkce pro praci s ceckovskymi retezci.
#include <string.h>

// Pokud chci pouzivat standardni objekty ze STL (napr. cout) a nechci
// pred jejich pouzitim stale psat namespace, tak pouziji prikaz using:
using namespace std;

// Hlavni funkce programu, musi se jmenovat main() a je spoustena jako
// prvni. Pokud ma byt vysledek prekladu spustitelny, musi ji obsahovat!
// (Pokud prekladame napr. dynamickou knihovnu, neni funkce potreba)
int main(){

	// Vytvori pole znaku o delce 10 bytu
    char memory[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
    // Spatna deklarace:
    // char memory[9] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
    // Prvku je vice nez se vejde do pole.
    // Tohle je spravne:
    // char memory[9] = {'0', '1', '2', '3', '4', '5', '6'}
    // Posledni tri prvky pole budou mit hodnotu 0

    // Vytvori ceckovsky retezec - pole znaku ukoncene bytem s hodnotou 0.
    // Delka retezce je tedy 10 + 1
    char retezec[] = "0123456789";

    // Deklarace dvou promennych ukazatel na char a promenne typu char na
    // jednom radku.
    char *ukazatel, *ukazatel2, promenna;

    // Deklarace dvourozmerneho pole typu integer s fixnim poctem prvku 3x3.
    int pole2D[3][3] = { {1, 2, 3}, {4, 5, 6}, {7, 8, 9}};
    // Lze deklarovat i takto:
    // int pole2D[3][3] = { 1, 2, 3, 4, 5, 6, 7, 8, 9};
    // Obe deklarace maji prakticky stejny vyznam. Prvky takto vytvoreneho
    // pole jsou stejne ulozeny do jednorozmerneho pole.

	// Promenna ukazatelInt nyni ukazuje na zacatek pole2D. Je nutne pretypovat,
    // ale v obou promennych je pouze adresa do pameti.
    int *ukazatelInt = (int *) pole2D;
    
    // Ukazatel je obycejna promenna, ktera obsahuje misto ciselne hodnoty
    // hodnotu adresy do pameti. Nasledujici promennou typu integer zkusime
    // pouzit jako ukazatel.
    int adresa;
    
    // Do promenne adresa zkopirujeme adresu promenne "promenna". Je treba
    // pretypovat, protoze u operatoru prirazeni mame na kazde strane zcela
    // jiny datovy typ.
    adresa = (int) &promenna;
    
    // Muzeme si dokonce primo vypsat na terminal, jak adresa vypada.
    cout << hex << "0x" << adresa << endl;
    
    // Do promenne ukazatel2 vlozime hodnotu promenne adresa. Je tam tedy
    // vlozena adresa promenne "promenna", jinymi slovy ukazatel2 nyni ukazuje
    // na promennou "promenna".
    ukazatel2 = (char *) adresa;
    
    // Do mista kam ukazuje ukazatel2 (to nam zpusobi ta hvezdicka), vlozime
    // hodnotu 100.
    *ukazatel2 = 100;
    
    // Provedeme kontrolu, zda se nam hodnota 100 opravdu dostala az do "promenna".
    // Pretypovani je zde proto, ze "promenna" je typu char a objekt cout tento
    // datovy typ tiskne automaticky jako ascii znak a ne jako cislo.
    cout << dec << (int) promenna << endl;
    
    // Promenna ukazatel ukazuje na zacatek pole retezec:
    ukazatel = retezec;
    // Pokud zadame promennou nejakeho pole bez indexu, tak pracujeme s jeji adresou,
    // tedy s ukazatelem na zacatek pameti, kde se pole nachazi.
    
    // Pro praci s ceckovskymi retezci slouzi funkce z hlavickoveho souboru string.h.
    // Nasledujici funkce vraci pocet znaku v retezci bez ukoncovaciho nuloveho bytu.
    cout << strlen(retezec) << endl;
    // Dalsi uzitecne funkce jsou napr. strcpy, strncpy, strcmp, strtok, atd...
    // Vice lze najit v manualu.
    
    // Pozor na castou chybu, pokud se pouzije pole znaku char a neni ukonceno nulovym
    // bytem, je chovani funkci ze string.h nedefinovano!!!
    cout << strlen(memory) << endl;
    // Predchozi radek je ze syntaktickeho hlediska v poradku, ale zcela chybny v semantice.
    // Promenna memory neni ceckovsky retezec!!!
    
    // Pokud chceme, aby promenna ukazatel ukazovala na urcity prvek v poli, provedeme to
    // takto:
    ukazatel = &memory[3];
    // Pokud za polem uvedeme index, tak vracime primo hodnotu, ktera ja na danem indexu
    // ulozena. Pokud chceme adresu tohoto prvku, musime si o to rict pomoci ampersandu.
    
    // Nakonec demostrace toho, ze vsechny hodnoty v dvoutozmernem poli jsou ulozeny
    // v jednorozmernem poli za sebou:
    ukazatel = (char *) pole2D;
    for(int i = 0; i < 9; ++i){
    	cout << (int) ukazatelInt[i] << endl;
    }
    
    return 0;
}
